/* tslint:disable:no-trailing-whitespace */
import {
  promises as fsPromises,
  writeFileSync,
} from "fs";
import {dedent} from "tslint/lib/utils";
import {parseString} from "xml2js";

const util = require("util");
const exec = util.promisify(require("child_process").exec);

async function createVersionsFile(filename: string): Promise<void> {
  let version = undefined; // process.env.npm_package_version;
  fsPromises.readFile("./pom.xml", "utf-8")
    .then((xml) => {
      parseString(xml, (err, result) => {
        if (err) {
          // tslint:disable-next-line:no-console
          console.log("There was an error when parsing the file pom.xml to json: " + err);
        } else {
          version = result.project.version;
        }
      });
    })
    .catch((reason) => {
      // tslint:disable-next-line:no-console
      console.log("There was some problem reading the file pom.xml: " + reason);
    });

  const revision = (await exec("git rev-parse --short HEAD")).stdout.toString().trim();
  const branch = (await exec("git rev-parse --abbrev-ref HEAD")).stdout.toString().trim();

  // tslint:disable-next-line
  console.log(`version: '${version}', revision: '${revision}', branch: '${branch}'`);

  const content = dedent`
      // this file is automatically generated by git.version.ts script
      export const versions: Versions = {
        version: "${version}",
        revision: "${revision}",
        branch: "${branch}"
      };
      
      export interface Versions {
        version: string;
        revision: string;
        branch: string;
      }
      `;

  writeFileSync(filename, content, {encoding: "utf8"});
}

createVersionsFile("src/environments/versions.ts");
