import {
  ChangeDetectionStrategy,
  Component,
  OnInit,
} from "@angular/core";
import {
  AbstractControl,
  FormBuilder,
  FormGroup,
  Validators,
} from "@angular/forms";
import {MatDialogRef} from "@angular/material/dialog";
import {environment} from "@environments/environment";
import {SharedAbstractContainer} from "@shared/components/containers/shared-abstract/shared-abstract.container";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {
  FormValidationHelper,
  PropertyName,
  SolidifyValidator,
} from "solidify-frontend";

@Component({
  selector: "dlcm-privacy-policy-terms-of-use-approval-dialog",
  templateUrl: "./privacy-policy-terms-of-use-approval.dialog.html",
  styleUrls: ["./privacy-policy-terms-of-use-approval.dialog.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class PrivacyPolicyTermsOfUseApprovalDialog extends SharedAbstractContainer implements OnInit {
  isLoading: boolean = false;
  form: FormGroup;
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  readonly privacyPolicyLink: string = environment.privacyPolicyLink;
  readonly termsOfUseLink: string = environment.termsOfUseLink;

  constructor(private readonly _dialogRef: MatDialogRef<PrivacyPolicyTermsOfUseApprovalDialog>,
              private readonly _fb: FormBuilder) {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.form = this._fb.group({
      [this.formDefinition.acceptPrivacyPolicy]: [false, [Validators.requiredTrue, SolidifyValidator]],
      [this.formDefinition.acceptTermsOfUse]: [false, [Validators.requiredTrue, SolidifyValidator]],
    });
  }

  close(): void {
    this._dialogRef.close();
  }

  onSubmit(): void {
    if (this.form.invalid) {
      return;
    }
    this.isLoading = true;
    this._dialogRef.close(true);
  }

  getFormControl(key: string): AbstractControl {
    return FormValidationHelper.getFormControl(this.form, key);
  }

  get formValidationHelper(): typeof FormValidationHelper {
    return FormValidationHelper;
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() acceptPrivacyPolicy: string;
  @PropertyName() acceptTermsOfUse: string;
}
