import {
  ChangeDetectionStrategy,
  Component,
  Inject,
  OnInit,
} from "@angular/core";
import {FormBuilder} from "@angular/forms";
import {
  MAT_DIALOG_DATA,
  MatDialogRef,
} from "@angular/material/dialog";
import {FileUploadWrapper} from "@app/features/deposit/models/file-upload-wrapper.model";
import {AbstractDepositFileUploadDialog} from "@deposit/components/dialogs/abstract-deposit-file-upload/abstract-deposit-file-upload.dialog";
import {Enums} from "@enums";
import {
  EnumUtil,
  isNotNullNorUndefined,
  isNullOrUndefined,
} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-file-upload-archive-dialog",
  templateUrl: "./deposit-file-upload-archive.dialog.html",
  styleUrls: ["./deposit-file-upload-archive.dialog.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositFileUploadArchiveDialog extends AbstractDepositFileUploadDialog implements OnInit {
  archiveToUpload: File;
  isFormValid: boolean = true;

  get enumsDataFile(): typeof Enums.DataFile {
    return Enums.DataFile;
  }

  get enumUtil(): typeof EnumUtil {
    return EnumUtil;
  }

  constructor(protected _dialogRef: MatDialogRef<DepositFileUploadArchiveDialog>,
              protected _fb: FormBuilder,
              @Inject(MAT_DIALOG_DATA) public data: DepositFileUploadArchiveDialogData) {
    super(_dialogRef, _fb);
  }

  ngOnInit(): void {
    super.ngOnInit();

    if (isNotNullNorUndefined(this.data.dataCategoryEnum)) {
      const dataCategoryFormControl = this.form.get(this.formDefinition.dataCategory);
      dataCategoryFormControl.setValue(this.data.dataCategoryEnum);
    }
  }

  onFileChange(event: Event & any): void {
    if (event.target.files.length > 0) {
      const files: FileList = event.target.files;
      this.archiveToUpload = files[0];
    }
  }

  onSubmit(): void {
    const base = this.form.value as FileUploadWrapper;
    const fileUploadWrapper = {} as FileUploadWrapper;
    Object.assign(fileUploadWrapper, base);
    fileUploadWrapper.file = this.archiveToUpload;
    this._dialogRef.close(fileUploadWrapper);
  }

  delete(file: File): void {
    this.archiveToUpload = undefined;
  }

  isFileNameInvalid(): boolean {
    if (isNullOrUndefined(this.archiveToUpload)) {
      return false;
    }
    const fileName = this.archiveToUpload.name;
    return this.REGEX_INVALID.test(fileName);
  }

  selectCategory(): void {
    this.isFormValid = false;
  }

  selectDataType(): void {
    this.isFormValid = true;
  }
}

export interface DepositFileUploadArchiveDialogData {
  dataCategoryEnum?: Enums.DataFile.DataCategoryEnum;
}
